/*
 * Syncany, www.syncany.org
 * Copyright (C) 2011 Philipp C. Heckel <philipp.heckel@gmail.com> 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.syncany.connection.plugins.local;

import org.syncany.connection.plugins.ConfigPanel;
import org.syncany.util.FileUtil;
import java.io.File;
import java.util.ResourceBundle;
import org.syncany.config.Config;

/**
 *
 * @author Philipp C. Heckel <philipp.heckel@gmail.com>
 */
public class LocalConfigPanel extends ConfigPanel {

	private ResourceBundle resourceBundle;
	
    /** Creates new form LocalConfigPanel */
    public LocalConfigPanel(LocalConnection connection) {
        super(connection);
        resourceBundle = Config.getInstance().getResourceBundle();
        initComponents();
    }

    @Override
    public void load() {
        if (getConnection().getFolder() != null) {
            txtLocalFolder.setText(getConnection().getFolder().getAbsolutePath());
        }
    }

    @Override
    public void save() {
        getConnection().setFolder(new File(txtLocalFolder.getText()));
    }

    @Override
    public LocalConnection getConnection() {
        return (LocalConnection) super.getConnection();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                jLabel1 = new javax.swing.JLabel();
                txtLocalFolder = new javax.swing.JTextField();
                btnBrowse = new javax.swing.JButton();

                jLabel1.setText(resourceBundle.getString("lcp_repository_folder"));
                jLabel1.setName("jLabel1"); // NOI18N

                txtLocalFolder.setName("txtLocalFolder"); // NOI18N

                btnBrowse.setText(resourceBundle.getString("lcp_browse"));
                btnBrowse.setName("btnBrowse"); // NOI18N
                btnBrowse.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                btnBrowseActionPerformed(evt);
                        }
                });

                javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
                this.setLayout(layout);
                layout.setHorizontalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel1)
                                        .addComponent(txtLocalFolder, javax.swing.GroupLayout.DEFAULT_SIZE, 359, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(btnBrowse, javax.swing.GroupLayout.PREFERRED_SIZE, 89, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap())
                );
                layout.setVerticalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addGap(21, 21, 21)
                                                .addComponent(btnBrowse))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel1)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(txtLocalFolder, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addContainerGap(24, Short.MAX_VALUE))
                );
        }// </editor-fold>//GEN-END:initComponents

    private void btnBrowseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBrowseActionPerformed
        File selectedFolder = FileUtil.showBrowseDirectoryDialog();

        if (selectedFolder != null) {
            txtLocalFolder.setText(selectedFolder.getAbsolutePath());
        }
    }//GEN-LAST:event_btnBrowseActionPerformed


        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JButton btnBrowse;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JTextField txtLocalFolder;
        // End of variables declaration//GEN-END:variables

}
