/*
 * Syncany, www.syncany.org
 * Copyright (C) 2011 Philipp C. Heckel <philipp.heckel@gmail.com> 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.syncany.gui.error;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.syncany.Constants;
import org.syncany.Syncany;
import org.syncany.config.Config;
import org.syncany.util.StringUtil;

/**
 *
 * @author pheckel
 */
public class ErrorDialog extends javax.swing.JDialog {
    private static final Logger logger = Logger.getLogger(ErrorDialog.class.getSimpleName());
    private ResourceBundle resourceBundle;

    private ErrorDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        resourceBundle = Config.getInstance().getResourceBundle();
        initComponents();

        lblSent.setVisible(false);
        getRootPane().setDefaultButton(btnReport);
        setLocationRelativeTo(null);
    }
    
    private void setThrowable(Throwable e) {	
    txtError.setText(StringUtil.getStackTrace(e));
    }
    
    public static void showDialog(Throwable e) {
        ErrorDialog dialog = new ErrorDialog(new JFrame(), true);
	
        dialog.setThrowable(e);	
        dialog.addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(java.awt.event.WindowEvent e) {
                System.exit(2);
            }
        });	

        dialog.setVisible(true);        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                jScrollPane1 = new javax.swing.JScrollPane();
                txtError = new javax.swing.JTextArea();
                btnRestart = new javax.swing.JButton();
                btnReport = new javax.swing.JButton();
                btnQuit = new javax.swing.JButton();
                jLabel3 = new javax.swing.JLabel();
                jLabel2 = new javax.swing.JLabel();
                jLabel1 = new javax.swing.JLabel();
                jScrollPane2 = new javax.swing.JScrollPane();
                txtComments = new javax.swing.JTextArea();
                jLabel4 = new javax.swing.JLabel();
                lblSent = new javax.swing.JLabel();

                setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
                setResizable(false);

                jScrollPane1.setName("jScrollPane1"); // NOI18N

                txtError.setColumns(20);
                txtError.setRows(5);
                txtError.setName("txtError"); // NOI18N
                jScrollPane1.setViewportView(txtError);

                btnRestart.setText(resourceBundle.getString("errd_restart_syncany"));
                btnRestart.setName("btnRestart"); // NOI18N
                btnRestart.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                btnRestartActionPerformed(evt);
                        }
                });

                btnReport.setText(resourceBundle.getString("errd_report_crash"));
                btnReport.setName("btnReport"); // NOI18N
                btnReport.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                btnReportActionPerformed(evt);
                        }
                });

                btnQuit.setText(resourceBundle.getString("errd_quit_syncany"));
                btnQuit.setName("btnQuit"); // NOI18N
                btnQuit.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                btnQuitActionPerformed(evt);
                        }
                });

                jLabel3.setText(resourceBundle.getString("errd_report_bug"));
                jLabel3.setName("jLabel3"); // NOI18N

                jLabel2.setText(resourceBundle.getString("errd_syncany_crashed"));
                jLabel2.setName("jLabel2"); // NOI18N

                jLabel1.setFont(jLabel1.getFont().deriveFont(jLabel1.getFont().getStyle() | java.awt.Font.BOLD, jLabel1.getFont().getSize()+2));
                jLabel1.setText(resourceBundle.getString("errd_message"));
                jLabel1.setName("jLabel1"); // NOI18N

                jScrollPane2.setName("jScrollPane2"); // NOI18N

                txtComments.setColumns(20);
                txtComments.setRows(5);
                txtComments.setName("txtComments"); // NOI18N
                jScrollPane2.setViewportView(txtComments);

                jLabel4.setText(resourceBundle.getString("errd_comments"));
                jLabel4.setName("jLabel4"); // NOI18N

                lblSent.setFont(lblSent.getFont().deriveFont(lblSent.getFont().getStyle() | java.awt.Font.BOLD));
                lblSent.setText(resourceBundle.getString("errd_report_was_sent"));
                lblSent.setName("lblSent"); // NOI18N

                javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
                getContentPane().setLayout(layout);
                layout.setHorizontalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                        .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 660, Short.MAX_VALUE)
                                        .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(btnReport, javax.swing.GroupLayout.PREFERRED_SIZE, 121, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(lblSent)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 87, Short.MAX_VALUE)
                                                .addComponent(btnQuit, javax.swing.GroupLayout.PREFERRED_SIZE, 121, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(btnRestart, javax.swing.GroupLayout.PREFERRED_SIZE, 121, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 660, Short.MAX_VALUE)
                                        .addComponent(jLabel4, javax.swing.GroupLayout.Alignment.LEADING))
                                .addContainerGap())
                );
                layout.setVerticalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jLabel2)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jLabel3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 241, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(jLabel4)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 84, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(btnRestart)
                                        .addComponent(btnReport)
                                        .addComponent(btnQuit)
                                        .addComponent(lblSent))
                                .addContainerGap())
                );

                pack();
        }// </editor-fold>//GEN-END:initComponents

    private void btnQuitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnQuitActionPerformed
        System.exit(2);
    }//GEN-LAST:event_btnQuitActionPerformed

    private void btnRestartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRestartActionPerformed
        dispose();
        Syncany.start();
    }//GEN-LAST:event_btnRestartActionPerformed

    private void btnReportActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnReportActionPerformed
        final ErrorDialog errDialog = this;

        btnReport.setText(resourceBundle.getString("errd_sending"));
        btnReport.setEnabled(false);

        new Thread(new Runnable() {
            @Override
            public void run() {
                HttpURLConnection httpCon = null;
                
                try {
                    // Start sending!
                    String queryString = "stack="+URLEncoder.encode(txtError.getText(), "UTF-8")
                    + "&comments="+URLEncoder.encode(txtComments.getText(), "UTF-8");

                    URL url = new URL(Constants.APPLICATION_CRASHREPORT_URL);	     
                    httpCon = (HttpURLConnection) url.openConnection();
                    httpCon.setDoOutput(true);
                    httpCon.setRequestMethod("POST");	     

                    OutputStreamWriter out = new OutputStreamWriter(httpCon.getOutputStream());
                    out.write(queryString);
                    out.flush();

                    BufferedReader in = new BufferedReader(new InputStreamReader(httpCon.getInputStream()));
                    String line = null;

                    while ((line = in.readLine()) != null) {
                        System.out.println(line);
                    }

                    out.close();
                    in.close(); 	    

                    if (httpCon.getResponseCode() != 200){		
                        throw new Exception("Something went wrong. Error code "+httpCon.getResponseCode()); 
                    }

                    btnReport.setText(resourceBundle.getString("errd_thank_you"));
                    lblSent.setVisible(true);

                    if (logger.isLoggable(Level.INFO)) {
                        logger.log(Level.INFO, "Crash report sent successfully. Code = {0}", httpCon.getResponseCode());
                        logger.log(Level.INFO, "Response message = {0}", httpCon.getResponseMessage());
                    }
                } 
                catch (Exception e) {
                    if (logger.isLoggable(Level.SEVERE)) {
                        if (httpCon != null) {
                            try {
                                logger.log(Level.SEVERE, "Sending crash report FAILED. Code = {0}", httpCon.getResponseCode());
                                logger.log(Level.SEVERE, "- Message = {0}", httpCon.getResponseMessage());
                            }
                            catch (Exception ee) { }
                        }
                        else {
                            logger.log(Level.SEVERE, "Sending crash report FAILED. No HTTP connection!");
                        }
                    }    
                    
                    btnReport.setText(resourceBundle.getString("errd_report_crash"));
                    btnReport.setEnabled(true);

                    JOptionPane.showMessageDialog(errDialog, resourceBundle.getString("email_bug_report"));
                }
            }
        }, "SendCrashRep").start();
    }//GEN-LAST:event_btnReportActionPerformed

        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JButton btnQuit;
        private javax.swing.JButton btnReport;
        private javax.swing.JButton btnRestart;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JLabel jLabel2;
        private javax.swing.JLabel jLabel3;
        private javax.swing.JLabel jLabel4;
        private javax.swing.JScrollPane jScrollPane1;
        private javax.swing.JScrollPane jScrollPane2;
        private javax.swing.JLabel lblSent;
        private javax.swing.JTextArea txtComments;
        private javax.swing.JTextArea txtError;
        // End of variables declaration//GEN-END:variables

}
