/*
 * Syncany, www.syncany.org
 * Copyright (C) 2011 Philipp C. Heckel <philipp.heckel@gmail.com> 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.syncany.gui.settings;

import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.syncany.Constants;
import org.syncany.config.Config;
import org.syncany.util.FileUtil;
import org.syncany.util.ImageUtil;

/**
 *
 * @author Philipp C. Heckel <philipp.heckel@gmail.com>
 */
public class ApplicationPanel extends SettingsPanel {
    private static final Logger logger = Logger.getLogger(ApplicationPanel.class.getSimpleName());
    private Image cachedUserImage;
    private ResourceBundle resourceBundle;
    
    /** Creates new form ApplicationPanel */
    public ApplicationPanel() {
    	resourceBundle = Config.getInstance().getResourceBundle();
        initComponents();
	
        lblUserImage.setText("");
    }

    @Override
    public void load() {
        chkStartup.setSelected(config.isAutostart());
        chkShowNotifications.setSelected(config.isNotificationsEnabled());
        txtName.setText(config.getUserName());
        txtMachineName.setText(config.getMachineName());
	
    // User Image
    btnImageBrowse.setEnabled(rdImageOther.isSelected());
	
    switch (config.getUserImageType()) {
        case None:
        rdImageNone.setSelected(true);		
        break;
		
        case Other:
        rdImageOther.setSelected(true);
        break;
		
        case System:
        default:
        rdImageSystem.setSelected(true);		
    }
	
    ImageIcon imageIcon = new ImageIcon(config.getUserImage());
	
    cachedUserImage = config.getUserImage(Config.UserImageType.Other);
    lblUserImage.setIcon(imageIcon);
    }

    @Override
    public void save() {
        config.setAutostart(chkStartup.isSelected());
        config.setNotificationsEnabled(chkShowNotifications.isSelected());
        config.setUserName(txtName.getText());
        config.setMachineName(txtMachineName.getText());
	
    // User Image
    if (rdImageNone.isSelected()) {
        config.setUserImageType(Config.UserImageType.None);
    }
	
    else if (rdImageSystem.isSelected()) {
        config.setUserImageType(Config.UserImageType.System);
    }
	
    else if (rdImageOther.isSelected()) {
        config.setUserImageType(Config.UserImageType.Other);
	    
        // Save image to 'profile.png'
        BufferedImage writeImage = ImageUtil.toBufferedImage(cachedUserImage);
        File profileImageFile = new File(config.getConfDir()+File.separator+"profile.png");

        try {
        ImageIO.write(writeImage, "png", profileImageFile);
        }
        catch (IOException e) {
        logger.warning("Unable to save image to "+profileImageFile);
        }
    }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                buttonGroup1 = new javax.swing.ButtonGroup();
                jLabel1 = new javax.swing.JLabel();
                jLabel5 = new javax.swing.JLabel();
                txtName = new javax.swing.JTextField();
                jLabel2 = new javax.swing.JLabel();
                chkStartup = new javax.swing.JCheckBox();
                chkShowNotifications = new javax.swing.JCheckBox();
                jLabel3 = new javax.swing.JLabel();
                txtMachineName = new javax.swing.JTextField();
                jLabel4 = new javax.swing.JLabel();
                jLabel6 = new javax.swing.JLabel();
                jLabel7 = new javax.swing.JLabel();
                jLabel8 = new javax.swing.JLabel();
                lblUserImage = new javax.swing.JLabel();
                jLabel9 = new javax.swing.JLabel();
                rdImageSystem = new javax.swing.JRadioButton();
                rdImageNone = new javax.swing.JRadioButton();
                rdImageOther = new javax.swing.JRadioButton();
                btnImageBrowse = new javax.swing.JButton();

                setName("Form"); // NOI18N
                setPreferredSize(new java.awt.Dimension(520, 390));

                jLabel1.setName("jLabel1"); // NOI18N

                jLabel5.setName("jLabel5"); // NOI18N

                txtName.setName("txtName"); // NOI18N

                jLabel2.setName("jLabel2"); // NOI18N

                chkStartup.setText(resourceBundle.getString("app_launch")); // NOI18N
                chkStartup.setName("chkStartup"); // NOI18N

                chkShowNotifications.setText(resourceBundle.getString("app_show_notifications")); // NOI18N
                chkShowNotifications.setName("chkShowNotifications"); // NOI18N

                jLabel3.setName("jLabel3"); // NOI18N

                txtMachineName.setName("txtMachineName"); // NOI18N

                jLabel4.setText(resourceBundle.getString("app_computer_name")); // NOI18N
                jLabel4.setName("jLabel4"); // NOI18N

                jLabel6.setText(resourceBundle.getString("app_username")); // NOI18N
                jLabel6.setName("jLabel6"); // NOI18N

                jLabel7.setFont(jLabel7.getFont().deriveFont(jLabel7.getFont().getStyle() | java.awt.Font.BOLD));
                jLabel7.setText(resourceBundle.getString("app_settings")); // NOI18N
                jLabel7.setName("jLabel7"); // NOI18N

                jLabel8.setFont(jLabel8.getFont().deriveFont(jLabel8.getFont().getStyle() | java.awt.Font.BOLD));
                jLabel8.setText(resourceBundle.getString("app_user_settings")); // NOI18N
                jLabel8.setName("jLabel8"); // NOI18N

                lblUserImage.setText(resourceBundle.getString("app_user_image")); // NOI18N
                lblUserImage.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
                lblUserImage.setName("lblUserImage"); // NOI18N

                jLabel9.setText(resourceBundle.getString("app_image")); // NOI18N
                jLabel9.setName("jLabel9"); // NOI18N

                buttonGroup1.add(rdImageSystem);
                rdImageSystem.setSelected(true);
                rdImageSystem.setText(resourceBundle.getString("app_use_system_profile_image")); // NOI18N
                rdImageSystem.setName("rdImageSystem"); // NOI18N
                rdImageSystem.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                rdImageActionPerformed(evt);
                        }
                });

                buttonGroup1.add(rdImageNone);
                rdImageNone.setText(resourceBundle.getString("app_no_image")); // NOI18N
                rdImageNone.setName("rdImageNone"); // NOI18N
                rdImageNone.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                rdImageActionPerformed(evt);
                        }
                });

                buttonGroup1.add(rdImageOther);
                rdImageOther.setText(resourceBundle.getString("app_user_other_image")); // NOI18N
                rdImageOther.setName("rdImageOther"); // NOI18N
                rdImageOther.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                rdImageActionPerformed(evt);
                        }
                });

                btnImageBrowse.setText(resourceBundle.getString("app_browse")); // NOI18N
                btnImageBrowse.setName("btnImageBrowse"); // NOI18N
                btnImageBrowse.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                btnImageBrowseActionPerformed(evt);
                        }
                });

                javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
                this.setLayout(layout);
                layout.setHorizontalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel1)
                                        .addComponent(jLabel3)
                                        .addComponent(jLabel5)
                                        .addComponent(jLabel2)
                                        .addGroup(layout.createSequentialGroup()
                                                .addContainerGap()
                                                .addComponent(jLabel7))
                                        .addGroup(layout.createSequentialGroup()
                                                .addContainerGap()
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(jLabel8)
                                                        .addComponent(jLabel6)
                                                        .addComponent(jLabel4)
                                                        .addComponent(jLabel9))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addGroup(layout.createSequentialGroup()
                                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                                        .addComponent(rdImageSystem)
                                                                        .addComponent(rdImageNone)
                                                                        .addGroup(layout.createSequentialGroup()
                                                                                .addComponent(rdImageOther)
                                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                                .addComponent(btnImageBrowse)))
                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 102, Short.MAX_VALUE)
                                                                .addComponent(lblUserImage, javax.swing.GroupLayout.PREFERRED_SIZE, 64, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                        .addComponent(txtMachineName, javax.swing.GroupLayout.DEFAULT_SIZE, 379, Short.MAX_VALUE)
                                                        .addComponent(txtName, javax.swing.GroupLayout.DEFAULT_SIZE, 379, Short.MAX_VALUE)))
                                        .addGroup(layout.createSequentialGroup()
                                                .addContainerGap()
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(chkShowNotifications)
                                                        .addComponent(chkStartup))))
                                .addContainerGap())
                );
                layout.setVerticalGroup(
                        layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel1)
                                        .addComponent(jLabel3)
                                        .addComponent(jLabel5)
                                        .addComponent(jLabel2))
                                .addGap(12, 12, 12)
                                .addComponent(jLabel7)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(chkStartup)
                                .addGap(7, 7, 7)
                                .addComponent(chkShowNotifications)
                                .addGap(21, 21, 21)
                                .addComponent(jLabel8)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(jLabel6)
                                        .addComponent(txtName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(txtMachineName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(18, 18, 18)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(rdImageNone)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(rdImageSystem)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                        .addComponent(rdImageOther)
                                                        .addComponent(btnImageBrowse)))
                                        .addComponent(jLabel9)
                                        .addComponent(lblUserImage, javax.swing.GroupLayout.PREFERRED_SIZE, 61, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(81, 81, 81))
                );
        }// </editor-fold>//GEN-END:initComponents

    private void btnImageBrowseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnImageBrowseActionPerformed
    File imageFile = FileUtil.showBrowseFileDialog();
	
    if (imageFile == null)
        return;
	
    ImageIcon imageIcon = new ImageIcon(imageFile.getAbsolutePath());
	
    if (imageIcon.getImageLoadStatus() != MediaTracker.COMPLETE) {
        JOptionPane.showMessageDialog(this, resourceBundle.getString("app_err_upload"));	    
        return;
    }
	
    cachedUserImage = ImageUtil.getScaledImage(imageIcon.getImage(), Constants.PROFILE_IMAGE_MAX_WIDTH, Constants.PROFILE_IMAGE_MAX_HEIGHT);
	
    imageIcon.setImage(cachedUserImage);
    lblUserImage.setIcon(imageIcon);
    }//GEN-LAST:event_btnImageBrowseActionPerformed

    private void rdImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rdImageActionPerformed
    btnImageBrowse.setEnabled(rdImageOther.isSelected());
	
    if (rdImageNone.isSelected()) {
        ImageIcon imageIcon = new ImageIcon(config.getUserImage(Config.UserImageType.None));
        lblUserImage.setIcon(imageIcon);
    }
	
    else if (rdImageSystem.isSelected()) {
        ImageIcon imageIcon = new ImageIcon(config.getUserImage(Config.UserImageType.System));
        lblUserImage.setIcon(imageIcon);
    }
	
    else if (rdImageOther.isSelected()) {
        lblUserImage.setIcon(new ImageIcon(cachedUserImage));	
    }			
    }//GEN-LAST:event_rdImageActionPerformed


        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JButton btnImageBrowse;
        private javax.swing.ButtonGroup buttonGroup1;
        private javax.swing.JCheckBox chkShowNotifications;
        private javax.swing.JCheckBox chkStartup;
        private javax.swing.JLabel jLabel1;
        private javax.swing.JLabel jLabel2;
        private javax.swing.JLabel jLabel3;
        private javax.swing.JLabel jLabel4;
        private javax.swing.JLabel jLabel5;
        private javax.swing.JLabel jLabel6;
        private javax.swing.JLabel jLabel7;
        private javax.swing.JLabel jLabel8;
        private javax.swing.JLabel jLabel9;
        private javax.swing.JLabel lblUserImage;
        private javax.swing.JRadioButton rdImageNone;
        private javax.swing.JRadioButton rdImageOther;
        private javax.swing.JRadioButton rdImageSystem;
        private javax.swing.JTextField txtMachineName;
        private javax.swing.JTextField txtName;
        // End of variables declaration//GEN-END:variables


}
