/*
 * Syncany, www.syncany.org
 * Copyright (C) 2011 Philipp C. Heckel <philipp.heckel@gmail.com> 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ProxyPanel.java
 *
 * Created on Mar 27, 2011, 12:51:42 AM
 */
package org.syncany.gui.settings;

import java.util.ResourceBundle;

import org.syncany.config.Config;
import org.syncany.config.Profile;
import org.syncany.config.Repository;
import org.syncany.connection.plugins.ConfigPanel;
import org.syncany.connection.plugins.Connection;

/**
 *
 * @author Philipp C. Heckel <philipp.heckel@gmail.com>
 */
public class RepositoryPanel extends SettingsPanel {

    private Profile profile;
    private ConfigPanel pnlConnection;
    private ResourceBundle resourceBundle;

    /** Creates new form ProxyPanel */
    public RepositoryPanel(Profile profile) {
        resourceBundle = Config.getInstance().getResourceBundle();
        initComponents();

        this.profile = profile;
        this.pnlConnection = profile.getRepository().getConnection().createConfigPanel();

        scrConnection.setViewportView(pnlConnection);
        //scrConnection.getViewport().setViewPosition(new Point(0, 0));

    }

    @Override
    public void load() {
        Repository repo = profile.getRepository();
        Connection connection = repo.getConnection();

        lblStorageType.setText(connection.getPluginInfo().getName() + ", " + resourceBundle.getString("reposp_version") + "  " + repo.getConnection().getPluginInfo().getVersionStr());
        lblStorageDescription.setText(connection.getPluginInfo().getDescripton());
        lblEncryption.setText(repo.getEncryption().getCipherStr() + ", " + repo.getEncryption().getKeylength() + " bit");

        lblConnection.setText(connection.getPluginInfo().getName() + resourceBundle.getString("reposp_connection_details"));
        pnlConnection.load();
        jspRemoteInterval.setValue(profile.getRemoteWatcher().getInterval());
    }

    @Override
    public void save() {
        // TODO
        pnlConnection.save();
        profile.getRemoteWatcher().setInterval((Integer) jspRemoteInterval.getValue());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        lblConnection = new javax.swing.JLabel();
        scrConnection = new javax.swing.JScrollPane();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        lblStorageType = new javax.swing.JLabel();
        lblEncryption = new javax.swing.JLabel();
        lblStorageDescription = new javax.swing.JLabel();
        jspRemoteInterval = new javax.swing.JSpinner();
        jLabel2 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();

        jLabel1.setFont(jLabel1.getFont().deriveFont(jLabel1.getFont().getStyle() | java.awt.Font.BOLD));
        jLabel1.setText("Remote Storage");
        jLabel1.setName("jLabel1"); // NOI18N

        lblConnection.setFont(lblConnection.getFont().deriveFont(lblConnection.getFont().getStyle() | java.awt.Font.BOLD));
        lblConnection.setText(resourceBundle.getString("reposp_connection"));
        lblConnection.setName("lblConnection"); // NOI18N

        scrConnection.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        scrConnection.setViewportBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        scrConnection.setName("scrConnection"); // NOI18N

        jLabel3.setText(resourceBundle.getString("reposp_connection_type"));
        jLabel3.setName("jLabel3"); // NOI18N

        jLabel4.setText(resourceBundle.getString("reposp_encryption"));
        jLabel4.setName("jLabel4"); // NOI18N

        lblStorageType.setText(resourceBundle.getString("reposp_temp_type"));
        lblStorageType.setName("lblStorageType"); // NOI18N

        lblEncryption.setText(resourceBundle.getString("reposp_temp_encryption"));
        lblEncryption.setName("lblEncryption"); // NOI18N

        lblStorageDescription.setText(resourceBundle.getString("reposp_temp_description"));
        lblStorageDescription.setName("lblStorageDescription"); // NOI18N

        jspRemoteInterval.setName("jspRemoteInterval"); // NOI18N

        jLabel2.setText(resourceBundle.getString("reposp_watch_interval"));
        jLabel2.setName("lblWatchInterval"); // NOI18N

        jLabel5.setText(resourceBundle.getString("reposp_watch_ms"));
        jLabel5.setName("lblMiliSeconds"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jLabel1))
                    .addComponent(scrConnection, javax.swing.GroupLayout.PREFERRED_SIZE, 499, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(lblConnection)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel2))
                        .addGap(50, 50, 50)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jspRemoteInterval)
                            .addComponent(lblEncryption, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel5))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addGap(30, 30, 30)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lblStorageType)
                            .addComponent(lblStorageDescription))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(lblStorageType))
                .addGap(2, 2, 2)
                .addComponent(lblStorageDescription)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(lblEncryption))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jspRemoteInterval, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 28, Short.MAX_VALUE)
                .addComponent(lblConnection)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(scrConnection, javax.swing.GroupLayout.PREFERRED_SIZE, 237, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JSpinner jspRemoteInterval;
    private javax.swing.JLabel lblConnection;
    private javax.swing.JLabel lblEncryption;
    private javax.swing.JLabel lblStorageDescription;
    private javax.swing.JLabel lblStorageType;
    private javax.swing.JScrollPane scrConnection;
    // End of variables declaration//GEN-END:variables
}
