/*
 * Syncany, www.syncany.org
 * Copyright (C) 2011 Philipp C. Heckel <philipp.heckel@gmail.com> 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.syncany.index.chunk;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.util.Enumeration;
import java.util.logging.Level;

/**
 *
 * @author Philipp C. Heckel <philipp.heckel@gmail.com>
 */
public class FixedChunker extends Chunker {
    private int chunkSize;
    private MessageDigest digest;
    
    
    /**
     * Chunk size in KB
     */
    public FixedChunker(int chunkSize) {
        this.chunkSize = chunkSize;        
 
        try {
            this.digest = MessageDigest.getInstance("SHA1");
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }        
    }
  
    @Override
    public Enumeration<Chunk> createChunks(File file) throws FileNotFoundException {
        return new FixedChunkEnumeration(file, chunkSize);
    }

    public class FixedChunkEnumeration implements Enumeration<Chunk> {
        private InputStream fis;           
        private byte[] buffer;
        private boolean closed;
        
        public FixedChunkEnumeration(File file, int chunkSize) throws FileNotFoundException {
            this.fis = new FileInputStream(file);
            this.buffer = new byte[chunkSize];
            this.closed = false;
        }
        
        @Override
        public boolean hasMoreElements() {
            if (closed) {
                return false;
            }
            
            try {
                //System.out.println("fis ="+fis.available());
                return fis.available() > 0;
            }
            catch (IOException ex) {
                if (logger.isLoggable(Level.WARNING)) {
                    logger.log(Level.WARNING, "Error while reading from file input stream.", ex);
                }
                
                return false;
            }
        }

        @Override
        public Chunk nextElement() {    
            try {
                int read = fis.read(buffer);
                
                if (read == -1) {
                    return null;
                }
                
                // Close if this was the last bytes
                if (fis.available() == 0) {
                    fis.close();
                    closed = true;
                }
                
                // Create chunk
                digest.reset();
                digest.update(buffer, 0, read);
                
                //byte[] chunkContents = (read == buffer.length) ? buffer : Arrays.copyOf(buffer, read);

                return new Chunk(digest.digest(), buffer, read, new byte[] {0});
            } 
            catch (IOException ex) {                
                logger.log(Level.SEVERE, "Error while retrieving next chunk.", ex);
                return null;
            }
        }
        
    }
}

