/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.syncany.index.chunk;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.GZIPOutputStream;
import org.bouncycastle.jce.provider.BouncyCastleProvider;



/**
 *
 * @author pheckel
 */
public class SyncanyMetachunking {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }
    
    private static List<File> recFileList(File f) {
        List<File> l = new ArrayList<File>();
        
        if (f.isFile()) {
            l.add(f);
        }
        
        File[] children = f.listFiles();
        
        if (children != null) {
            for (File f2 : children) {
                l.addAll(recFileList(f2));
            }
        }
        
        return l;
    }
 
    public static void metachunks(List<File> files) throws FileNotFoundException, IOException {        
        Chunker c;
        //c = new TTTDChunker(128*1024, 256*1024, 80, 40);        
        c = new TTTDChunker(8*1024, 16*1024, 100, 50);        
        //c = new FixedChunker(100);
        CustomMetaChunk mc = null;

        int i=0;
        for (File file : files) {
            System.err.println("file "+file);
            Enumeration<Chunk> chunks = c.createChunks(file);
            
            while (chunks.hasMoreElements()) {
                Chunk chunk = chunks.nextElement();      
                
                if (mc != null && mc.isFull()) {
                    mc.close();
                    mc = null;
                }
                
                if (mc == null) {
                    i++;
                    mc = new CustomMetaChunk(512*1024, new GZIPOutputStream(new FileOutputStream(new File("/home/pheckel/Downloads/TEST/result/Metachunk-"+i))));                    
                }     
                
                mc.write(chunk);
            }
        }    
        
        if (mc != null) {
            mc.close();
        }
    }
    
    public static void tar(List<File> files) throws FileNotFoundException, IOException {        
        Chunker c;
        //c = new TTTDChunker(128*1024, 256*1024, 80, 40);        
        c = new TTTDChunker(8*1024, 16*1024, 100, 50, TTTDChunker.DEFAULT_WINDOW_SIZE, "MD4");        
        //c = new FixedChunker(100);
        TarMetaChunk mc = null;
        
        int i=0;
        for (File file : files) {
            //System.err.println("file "+file);
            Enumeration<Chunk> chunks = c.createChunks(file);
            
            while (chunks.hasMoreElements()) {
                Chunk chunk = chunks.nextElement();      
                
                if (mc != null && mc.isFull()) {
                    mc.close();
                    mc = null;
                }
                
                if (mc == null) {
                    i++;
                    mc = new TarMetaChunk(512*1024, new GZIPOutputStream(new FileOutputStream(new File("/home/pheckel/Downloads/TEST/result/Metachunk-"+i))));                    
                }     
                
                mc.write(chunk);
            }
        }    
        
        if (mc != null) {
            mc.close();
        }
    }        
    
    public static void fixedsize() throws FileNotFoundException, IOException {
        //Chunker c = new TTTDChunker(8*1024, 16*1024, 100, 50);        
        Chunker c = new FixedChunker(512);
        
        Enumeration<Chunk> chunks = c.createChunks(new File("/home/pheckel/Downloads/TEST/movies/A"));
        Chunk chunk = null;

        int i = -1;
        while (chunks.hasMoreElements()) {
            i++;
            
            chunk = chunks.nextElement();
            //System.out.println(chunk.getChecksum() + " - "+chunk.getSize());
            writeFile(chunk.getContents(), (int) chunk.getSize(), new File("/home/pheckel/Downloads/TEST/result/B-"+i));
        }             
    }
    
    public static void tttdtest() throws FileNotFoundException, IOException, NoSuchAlgorithmException {
        Chunker c = new TTTDChunker(2*1024, 4*1024, 100, 50, TTTDChunker.DEFAULT_WINDOW_SIZE, "MD4");        
        
        Enumeration<Chunk> chunks = c.createChunks(new File("/home/pheckel/Downloads/TEST/B"));
        Chunk chunk = null;

        int i = -1;
        while (chunks.hasMoreElements()) {
            i++;
            
            chunk = chunks.nextElement();
            System.out.println(toHex(chunk.getChecksum()) + " - "+chunk.getSize());
            writeFile(chunk.getContents(), (int) chunk.getSize(), new File("/home/pheckel/Downloads/TEST/result/B-"+i));
        }          
        
        
        System.out.println("file checksum = "+ toHex(chunk.getFileChecksum()));                
        
    }
    
    public static void main(String[] args) throws FileNotFoundException, IOException, NoSuchAlgorithmException {
        List<File> files;
        //files = recFileList(new File("/home/pheckel/Office"));
        //files = recFileList(new File("/home/pheckel/Downloads/seen"));
        files = recFileList(new File("/home/pheckel/Syncany/Philipp's PC/Pictures"));
        
        System.out.println("go");
        //fixedsize();
        //metachunks(files);
        tar(files);
        //tttdtest();    
        
    }
    
    public static void writeFile(byte[] bytes, int len, File file) throws IOException {
        FileOutputStream fos = new FileOutputStream(file);

        fos.write(bytes, 0, len);
        fos.close();
    } 
    
   public static void writeFile(InputStream is, File file) throws IOException {
        FileOutputStream fos = new FileOutputStream(file);

        int read = 0;
        byte[] bytes = new byte[4096];

        while ((read = is.read(bytes)) != -1) {
            fos.write(bytes, 0, read);
        }

        is.close();
        fos.close();
    }  
   
    public static String toHex(byte[] bytes) {
        BigInteger bi = new BigInteger(1, bytes);
        return String.format("%0" + (bytes.length << 1) + "x", bi);
    }      
    
}
